import marked from 'marked';
import { Observable } from 'rxjs';
import { catchError, map } from 'rxjs/operators';
import { Injectable, Component, ElementRef, Inject, Input, PLATFORM_ID, ViewEncapsulation, NgModule } from '@angular/core';
import { HttpClient, HttpResponse, HttpClientModule } from '@angular/common/http';
import { highlightAll } from 'prismjs';
import { isPlatformBrowser, CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// function clearHTML(text: string) {
//   return text.replace('', '');
// }
/** @type {?} */
const DEFAULT_OPTION = {
    gfm: true,
    tables: true,
    breaks: false,
    pedantic: false,
    sanitize: false,
    smartLists: true,
    smartypants: false
};
class NtMarkdownService {
    /**
     * @param {?} http
     */
    constructor(http) {
        this.http = http;
        this._renderer = new marked.Renderer();
        this.extendRenderer();
        this.setMarkedOptions({});
    }
    // get the content from remote resource
    /**
     * @param {?} path
     * @return {?}
     */
    getContent(path) {
        return this.http.get(path)
            .pipe(map(this.extractData), catchError(this.handleError));
    }
    /**
     * @return {?}
     */
    get renderer() {
        return this._renderer;
    }
    // handle data
    /**
     * @param {?} res
     * @return {?}
     */
    extractData(res) {
        return res.body || '';
    }
    /**
     * @param {?} options
     * @return {?}
     */
    setMarkedOptions(options) {
        options = Object.assign({}, DEFAULT_OPTION, options);
        options.renderer = this._renderer;
        marked.setOptions(options);
    }
    // comple markdown to html
    /**
     * @param {?} data
     * @return {?}
     */
    compile(data) {
        return marked(data);
    }
    // handle error
    /**
     * @private
     * @param {?} error
     * @return {?}
     */
    handleError(error) {
        /** @type {?} */
        let errMsg;
        if (error instanceof HttpResponse) {
            /** @type {?} */
            const body = error.body || '';
            /** @type {?} */
            const err = body.error || JSON.stringify(body);
            errMsg = `${error.status} - ${error.statusText || ''} ${err}`;
        }
        else {
            errMsg = error.message ? error.message : error.toString();
        }
        return Observable.throw(errMsg);
    }
    // extend marked render to support todo checkbox
    /**
     * @private
     * @return {?}
     */
    extendRenderer() {
        // this._renderer.image = function (href: string, title: string, text: string) {
        // 	return `<img href="${href}" alt="${title}">`;
        // };
        this._renderer.heading = (/**
         * @param {?} text
         * @param {?} level
         * @return {?}
         */
        function (text, level) {
            text = text.trim();
            return `<h${level} id="${text.replace(/\<a[^>]*?\>((.|\n)*?)\<\/a\>/g, '$1').replace(/ +/g, '-')}">
						${text}
					</h${level}>`;
        });
        this._renderer.listitem = (/**
         * @param {?} text
         * @return {?}
         */
        function (text) {
            if (/^\s*\[[x ]\]\s*/.test(text)) {
                /** @type {?} */
                const checkedStyle = 'vertical-align: middle; margin: 0 0.2em 0.25em -1.6em; font-size: 16px;';
                text = text
                    .replace(/^\s*\[ \]\s*/, `<input type="checkbox" style="${checkedStyle}" disabled>`)
                    .replace(/^\s*\[x\]\s*/, `<input type="checkbox" style="${checkedStyle}" checked disabled>`);
                return `<li style="list-style: none">${text}</li>`;
            }
            else {
                return `<li>${text}</li>`;
            }
        });
    }
}
NtMarkdownService.decorators = [
    { type: Injectable },
];
/** @nocollapse */
NtMarkdownService.ctorParameters = () => [
    { type: HttpClient }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NtMarkdownBlockComponent {
    /**
     * @param {?} _platformId
     * @param {?} _markdownService
     * @param {?} _elementRef
     */
    constructor(_platformId, _markdownService, _elementRef) {
        this._platformId = _platformId;
        this._markdownService = _markdownService;
        this._elementRef = _elementRef;
        this.changeLog = [];
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        /** @type {?} */
        const pathChange = changes.path;
        /** @type {?} */
        const dataChange = changes.data;
        if (pathChange && !pathChange.firstChange) {
            this.onPathChange();
        }
        if (dataChange && !dataChange.firstChange) {
            this.onDataChange();
        }
    }
    /**
     * @return {?}
     */
    get path() { return this._path; }
    /**
     * @param {?} value
     * @return {?}
     */
    set path(value) { this._path = value || ''; }
    /**
     * @return {?}
     */
    get data() { return this._data; }
    /**
     * @param {?} value
     * @return {?}
     */
    set data(value) { this._data = value; }
    // on input
    /**
     * @return {?}
     */
    onDataChange() {
        if (this.data) {
            this._elementRef.nativeElement.innerHTML = this._markdownService.compile(this.data);
        }
        else {
            this._elementRef.nativeElement.innerHTML = '';
        }
        this.postpare();
    }
    /**
     *  After view init
     * @return {?}
     */
    ngAfterViewInit() {
        if (this.path) {
            this.onPathChange();
        }
        else {
            this.onDataChange();
        }
    }
    /**
     * get remote conent;
     * @return {?}
     */
    onPathChange() {
        if (this.path) {
            this._ext = this.path && this.path.split('.').splice(-1).join();
            this._markdownService.getContent(this.path)
                .subscribe((/**
             * @param {?} data
             * @return {?}
             */
            data => {
                this._md = this._ext !== 'md' ? '```' + this._ext + '\n' + data + '\n```' : data;
                this._elementRef.nativeElement.innerHTML = this._markdownService.compile(this.prepare(this._md));
                this.postpare();
            }), (/**
             * @param {?} error
             * @return {?}
             */
            error => this._handleError(error)));
        }
    }
    /**
     * catch http error
     * @private
     * @param {?} error
     * @return {?}
     */
    _handleError(error) {
        return Promise.reject(error.message || error);
    }
    /**
     * Prepare string
     * @param {?} raw
     * @return {?}
     */
    prepare(raw) {
        if (!raw) {
            return '';
        }
        if (this._ext === 'md' || !this.path) {
            /** @type {?} */
            let isCodeBlock = false;
            return raw.split('\n').map((/**
             * @param {?} line
             * @return {?}
             */
            (line) => {
                if (this._trimLeft(line).substring(0, 3) === "```") {
                    isCodeBlock = !isCodeBlock;
                }
                return isCodeBlock ? line : line.trim();
            })).join('\n');
        }
        return raw.replace(/\"/g, '\'');
    }
    /**
     * @return {?}
     */
    postpare() {
        // 判断是否是浏览器执行，如果是：增加代码高亮
        typeof highlightAll === 'function'
            && isPlatformBrowser(this._platformId)
            && highlightAll(false);
    }
    /**
     * Trim left whitespace
     * @private
     * @param {?} line
     * @return {?}
     */
    _trimLeft(line) {
        return line.replace(/^\s+|\s+$/g, '');
    }
}
NtMarkdownBlockComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-markdown-block, [nt-markdown-block]',
                template: '<div [innerHtml]="markdown"></div>',
                encapsulation: ViewEncapsulation.None,
                host: {
                    class: 'nt-markdown-block'
                }
            },] },
];
/** @nocollapse */
NtMarkdownBlockComponent.ctorParameters = () => [
    { type: Object, decorators: [{ type: Inject, args: [PLATFORM_ID,] }] },
    { type: NtMarkdownService },
    { type: ElementRef }
];
NtMarkdownBlockComponent.propDecorators = {
    path: [{ type: Input }],
    data: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NtMarkdownBlockModule {
}
NtMarkdownBlockModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule, HttpClientModule],
                declarations: [NtMarkdownBlockComponent],
                providers: [NtMarkdownService],
                exports: [NtMarkdownBlockComponent],
                entryComponents: [NtMarkdownBlockComponent]
            },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NtMarkdownBlockModule, NtMarkdownBlockComponent, NtMarkdownService };
//# sourceMappingURL=markdown-block.js.map
