import marked from 'marked';
import { Observable } from 'rxjs';
import { catchError, map } from 'rxjs/operators';
import { Injectable, Component, ElementRef, Inject, Input, PLATFORM_ID, ViewEncapsulation, NgModule } from '@angular/core';
import { HttpClient, HttpResponse, HttpClientModule } from '@angular/common/http';
import { highlightAll } from 'prismjs';
import { isPlatformBrowser, CommonModule } from '@angular/common';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// function clearHTML(text: string) {
//   return text.replace('', '');
// }
/** @type {?} */
var DEFAULT_OPTION = {
    gfm: true,
    tables: true,
    breaks: false,
    pedantic: false,
    sanitize: false,
    smartLists: true,
    smartypants: false
};
var NtMarkdownService = /** @class */ (function () {
    function NtMarkdownService(http) {
        this.http = http;
        this._renderer = new marked.Renderer();
        this.extendRenderer();
        this.setMarkedOptions({});
    }
    // get the content from remote resource
    // get the content from remote resource
    /**
     * @param {?} path
     * @return {?}
     */
    NtMarkdownService.prototype.getContent = 
    // get the content from remote resource
    /**
     * @param {?} path
     * @return {?}
     */
    function (path) {
        return this.http.get(path)
            .pipe(map(this.extractData), catchError(this.handleError));
    };
    Object.defineProperty(NtMarkdownService.prototype, "renderer", {
        get: /**
         * @return {?}
         */
        function () {
            return this._renderer;
        },
        enumerable: true,
        configurable: true
    });
    // handle data
    // handle data
    /**
     * @param {?} res
     * @return {?}
     */
    NtMarkdownService.prototype.extractData = 
    // handle data
    /**
     * @param {?} res
     * @return {?}
     */
    function (res) {
        return res.body || '';
    };
    /**
     * @param {?} options
     * @return {?}
     */
    NtMarkdownService.prototype.setMarkedOptions = /**
     * @param {?} options
     * @return {?}
     */
    function (options) {
        options = __assign({}, DEFAULT_OPTION, options);
        options.renderer = this._renderer;
        marked.setOptions(options);
    };
    // comple markdown to html
    // comple markdown to html
    /**
     * @param {?} data
     * @return {?}
     */
    NtMarkdownService.prototype.compile = 
    // comple markdown to html
    /**
     * @param {?} data
     * @return {?}
     */
    function (data) {
        return marked(data);
    };
    // handle error
    // handle error
    /**
     * @private
     * @param {?} error
     * @return {?}
     */
    NtMarkdownService.prototype.handleError = 
    // handle error
    /**
     * @private
     * @param {?} error
     * @return {?}
     */
    function (error) {
        /** @type {?} */
        var errMsg;
        if (error instanceof HttpResponse) {
            /** @type {?} */
            var body = error.body || '';
            /** @type {?} */
            var err = body.error || JSON.stringify(body);
            errMsg = error.status + " - " + (error.statusText || '') + " " + err;
        }
        else {
            errMsg = error.message ? error.message : error.toString();
        }
        return Observable.throw(errMsg);
    };
    // extend marked render to support todo checkbox
    // extend marked render to support todo checkbox
    /**
     * @private
     * @return {?}
     */
    NtMarkdownService.prototype.extendRenderer = 
    // extend marked render to support todo checkbox
    /**
     * @private
     * @return {?}
     */
    function () {
        // this._renderer.image = function (href: string, title: string, text: string) {
        // 	return `<img href="${href}" alt="${title}">`;
        // };
        this._renderer.heading = (/**
         * @param {?} text
         * @param {?} level
         * @return {?}
         */
        function (text, level) {
            text = text.trim();
            return "<h" + level + " id=\"" + text.replace(/\<a[^>]*?\>((.|\n)*?)\<\/a\>/g, '$1').replace(/ +/g, '-') + "\">\n\t\t\t\t\t\t" + text + "\n\t\t\t\t\t</h" + level + ">";
        });
        this._renderer.listitem = (/**
         * @param {?} text
         * @return {?}
         */
        function (text) {
            if (/^\s*\[[x ]\]\s*/.test(text)) {
                /** @type {?} */
                var checkedStyle = 'vertical-align: middle; margin: 0 0.2em 0.25em -1.6em; font-size: 16px;';
                text = text
                    .replace(/^\s*\[ \]\s*/, "<input type=\"checkbox\" style=\"" + checkedStyle + "\" disabled>")
                    .replace(/^\s*\[x\]\s*/, "<input type=\"checkbox\" style=\"" + checkedStyle + "\" checked disabled>");
                return "<li style=\"list-style: none\">" + text + "</li>";
            }
            else {
                return "<li>" + text + "</li>";
            }
        });
    };
    NtMarkdownService.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    NtMarkdownService.ctorParameters = function () { return [
        { type: HttpClient }
    ]; };
    return NtMarkdownService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var NtMarkdownBlockComponent = /** @class */ (function () {
    function NtMarkdownBlockComponent(_platformId, _markdownService, _elementRef) {
        this._platformId = _platformId;
        this._markdownService = _markdownService;
        this._elementRef = _elementRef;
        this.changeLog = [];
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    NtMarkdownBlockComponent.prototype.ngOnChanges = /**
     * @param {?} changes
     * @return {?}
     */
    function (changes) {
        /** @type {?} */
        var pathChange = changes.path;
        /** @type {?} */
        var dataChange = changes.data;
        if (pathChange && !pathChange.firstChange) {
            this.onPathChange();
        }
        if (dataChange && !dataChange.firstChange) {
            this.onDataChange();
        }
    };
    Object.defineProperty(NtMarkdownBlockComponent.prototype, "path", {
        get: /**
         * @return {?}
         */
        function () { return this._path; },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) { this._path = value || ''; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NtMarkdownBlockComponent.prototype, "data", {
        get: /**
         * @return {?}
         */
        function () { return this._data; },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) { this._data = value; },
        enumerable: true,
        configurable: true
    });
    // on input
    // on input
    /**
     * @return {?}
     */
    NtMarkdownBlockComponent.prototype.onDataChange = 
    // on input
    /**
     * @return {?}
     */
    function () {
        if (this.data) {
            this._elementRef.nativeElement.innerHTML = this._markdownService.compile(this.data);
        }
        else {
            this._elementRef.nativeElement.innerHTML = '';
        }
        this.postpare();
    };
    /**
     *  After view init
     */
    /**
     *  After view init
     * @return {?}
     */
    NtMarkdownBlockComponent.prototype.ngAfterViewInit = /**
     *  After view init
     * @return {?}
     */
    function () {
        if (this.path) {
            this.onPathChange();
        }
        else {
            this.onDataChange();
        }
    };
    /**
     * get remote conent;
     */
    /**
     * get remote conent;
     * @return {?}
     */
    NtMarkdownBlockComponent.prototype.onPathChange = /**
     * get remote conent;
     * @return {?}
     */
    function () {
        var _this = this;
        if (this.path) {
            this._ext = this.path && this.path.split('.').splice(-1).join();
            this._markdownService.getContent(this.path)
                .subscribe((/**
             * @param {?} data
             * @return {?}
             */
            function (data) {
                _this._md = _this._ext !== 'md' ? '```' + _this._ext + '\n' + data + '\n```' : data;
                _this._elementRef.nativeElement.innerHTML = _this._markdownService.compile(_this.prepare(_this._md));
                _this.postpare();
            }), (/**
             * @param {?} error
             * @return {?}
             */
            function (error) { return _this._handleError(error); }));
        }
    };
    /**
     * catch http error
     */
    /**
     * catch http error
     * @private
     * @param {?} error
     * @return {?}
     */
    NtMarkdownBlockComponent.prototype._handleError = /**
     * catch http error
     * @private
     * @param {?} error
     * @return {?}
     */
    function (error) {
        return Promise.reject(error.message || error);
    };
    /**
     * Prepare string
     */
    /**
     * Prepare string
     * @param {?} raw
     * @return {?}
     */
    NtMarkdownBlockComponent.prototype.prepare = /**
     * Prepare string
     * @param {?} raw
     * @return {?}
     */
    function (raw) {
        var _this = this;
        if (!raw) {
            return '';
        }
        if (this._ext === 'md' || !this.path) {
            /** @type {?} */
            var isCodeBlock_1 = false;
            return raw.split('\n').map((/**
             * @param {?} line
             * @return {?}
             */
            function (line) {
                if (_this._trimLeft(line).substring(0, 3) === "```") {
                    isCodeBlock_1 = !isCodeBlock_1;
                }
                return isCodeBlock_1 ? line : line.trim();
            })).join('\n');
        }
        return raw.replace(/\"/g, '\'');
    };
    /**
     * @return {?}
     */
    NtMarkdownBlockComponent.prototype.postpare = /**
     * @return {?}
     */
    function () {
        // 判断是否是浏览器执行，如果是：增加代码高亮
        typeof highlightAll === 'function'
            && isPlatformBrowser(this._platformId)
            && highlightAll(false);
    };
    /**
     * Trim left whitespace
     */
    /**
     * Trim left whitespace
     * @private
     * @param {?} line
     * @return {?}
     */
    NtMarkdownBlockComponent.prototype._trimLeft = /**
     * Trim left whitespace
     * @private
     * @param {?} line
     * @return {?}
     */
    function (line) {
        return line.replace(/^\s+|\s+$/g, '');
    };
    NtMarkdownBlockComponent.decorators = [
        { type: Component, args: [{
                    selector: 'nt-markdown-block, [nt-markdown-block]',
                    template: '<div [innerHtml]="markdown"></div>',
                    encapsulation: ViewEncapsulation.None,
                    host: {
                        class: 'nt-markdown-block'
                    }
                },] },
    ];
    /** @nocollapse */
    NtMarkdownBlockComponent.ctorParameters = function () { return [
        { type: Object, decorators: [{ type: Inject, args: [PLATFORM_ID,] }] },
        { type: NtMarkdownService },
        { type: ElementRef }
    ]; };
    NtMarkdownBlockComponent.propDecorators = {
        path: [{ type: Input }],
        data: [{ type: Input }]
    };
    return NtMarkdownBlockComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var NtMarkdownBlockModule = /** @class */ (function () {
    function NtMarkdownBlockModule() {
    }
    NtMarkdownBlockModule.decorators = [
        { type: NgModule, args: [{
                    imports: [CommonModule, HttpClientModule],
                    declarations: [NtMarkdownBlockComponent],
                    providers: [NtMarkdownService],
                    exports: [NtMarkdownBlockComponent],
                    entryComponents: [NtMarkdownBlockComponent]
                },] },
    ];
    return NtMarkdownBlockModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NtMarkdownBlockModule, NtMarkdownBlockComponent, NtMarkdownService };
//# sourceMappingURL=markdown-block.js.map
